 /*
* aes128.h
*
* 128 bit AES encryption and decryption
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


 
#ifndef _AES128_H_
#define _AES128_H_

/*
    Some of the common modes of operation of AES encryption:
        ECB - Electronic codebook (basic AES encryption)
        CBC - Cipher-block chaining
        OFB - Output feedback
        CFB - Cipher feedback
        CTR - Counter (also known as integer counter mode (ICM)
              and segmented integer counter (SIC) mode)
        GCM - Galois counter mode
*/

#define AES_BLOCK_SIZE          16

#define AES_KEY_SIZE            16
#define AES_WORKING_KEY_SIZE    176

void aes128_expand_key(uint8_t working_key[AES_WORKING_KEY_SIZE], const uint8_t key[AES_KEY_SIZE]);
void aes128_encrypt(uint8_t state[AES_BLOCK_SIZE], const uint8_t working_key[AES_WORKING_KEY_SIZE]);
void aes128_decrypt(uint8_t state[AES_BLOCK_SIZE], const uint8_t working_key[AES_WORKING_KEY_SIZE]);

#endif
