 /*
* aes128_cbc.c
*
* 128 AES CBC mode
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


 
#include <inttypes.h>
#include <string.h>
#if defined(__MSP430__)
#include <msp430.h>
#endif

#include "aes128.h"
#include "aes128_cbc.h"

#if defined(AES128_WITH_CBC)
#if defined(AES128_WITH_ENCRYPT)
void aes128_encrypt_cbc(uint8_t state[16], uint8_t prev_state[16], const uint8_t working_key[176])
{
    int i;

    for (i = 0;  i < 16;  i++)
        state[i] ^= prev_state[i];
    aes128_encrypt(state, working_key);
    memcpy(prev_state, state, 16);
}
/*- End of function --------------------------------------------------------*/
#endif

#if defined(AES128_WITH_DECRYPT)
void aes128_decrypt_cbc(uint8_t state[16], uint8_t prev_state[16], const uint8_t working_key[176])
{
    int i;
    uint8_t save[16];

    memcpy(save, state, 16);
    aes128_decrypt(state, working_key);
    for (i = 0;  i < 16;  i++)
        state[i] ^= prev_state[i];
    memcpy(prev_state, save, 16);
}
/*- End of function --------------------------------------------------------*/
#endif
#endif
/*- End of file ------------------------------------------------------------*/
