/*
* aes128_gcm_galois.c
*
* Galois field multiply for 128 bit AES GCM mode
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

 
 
#include <inttypes.h>
#include <string.h>

#include "aes128.h"

#if !defined(__inline__)
#define __inline__ inline
#endif

/* XOR a vector into another vector. */
static __inline__ void vec_xor(uint8_t z[], const uint8_t x[], int n)
{
    int i;

    for (i = 0;  i < n;  i++)
        z[i] ^= x[i];
}

/* Shift an entire vector right by one bit. */
static __inline__ void vec_shr(uint8_t z[], int len)
{
    uint8_t zz1;
    uint8_t zz2;
    int i;

    zz1 = 0;
    for (i = 0;  i < len;  i++)
    {
        zz2 = (z[i] & 1)  ?  0x80  :  0;
        z[i] = zz1 | (z[i] >> 1);
        zz1 = zz2;
    }
}

/* Do the Galois field multiplication. */
void gcm_galois_mult(uint8_t z[], const uint8_t x[])
{
    uint8_t t[AES_BLOCK_SIZE];
    uint8_t v[AES_BLOCK_SIZE];
    int i;
    int j;
    uint8_t zz;
    uint8_t vv;

    /* Use a copy of the input vector, so we don't corrupt the original. */
    memcpy(v, x, AES_BLOCK_SIZE);
    memset(t, 0, AES_BLOCK_SIZE);
    for (i = 0;  i < 16;  i++)
    {
        zz = z[i];
        for (j = 0;  j < 8;  j++)
        {
            if ((zz & 0x80))
                vec_xor(t, v, AES_BLOCK_SIZE);
            vv = v[AES_BLOCK_SIZE - 1] & 0x01;
            vec_shr(v, AES_BLOCK_SIZE);
            if (vv)
                v[0] ^= 0xE1;
            zz <<= 1;
        }
    }
    memcpy(z, t, AES_BLOCK_SIZE);
}
