 /*
* aes128_tests.c
*
* CBC (Chain block cypher) AES128 code */
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


 
/* CBC (Chain block cypher) AES128 code */

#include <inttypes.h>
#include <stdio.h>
#include <string.h>
#include <msp430.h>

#include "aes128.h"
#include "aes128_cbc.h"

/* Test cases from RFC3602 */

/* Case #1: Encrypting 16 bytes (1 block) using AES-CBC with 128-bit key */
uint8_t key1[] = {0x06, 0xa9, 0x21, 0x40, 0x36, 0xb8, 0xa1, 0x5b, 0x51, 0x2e, 0x03, 0xd5, 0x34, 0x12, 0x00, 0x06};
uint8_t iv1[] = {0x3d, 0xaf, 0xba, 0x42, 0x9d, 0x9e, 0xb4, 0x30, 0xb4, 0x22, 0xda, 0x80, 0x2c, 0x9f, 0xac, 0x41};
uint8_t plain1[] = "Single block msg";
uint8_t cipher1[] = {0xe3, 0x53, 0x77, 0x9c, 0x10, 0x79, 0xae, 0xb8, 0x27, 0x08, 0x94, 0x2d, 0xbe, 0x77, 0x18, 0x1a};

/* Case #2: Encrypting 32 bytes (2 blocks) using AES-CBC with 128-bit key */
uint8_t key2[] = {0xc2, 0x86, 0x69, 0x6d, 0x88, 0x7c, 0x9a, 0xa0, 0x61, 0x1b, 0xbb, 0x3e, 0x20, 0x25, 0xa4, 0x5a};
uint8_t iv2[] = {0x56, 0x2e, 0x17, 0x99, 0x6d, 0x09, 0x3d, 0x28, 0xdd, 0xb3, 0xba, 0x69, 0x5a, 0x2e, 0x6f, 0x58};
uint8_t plain2[] =
{
    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
    0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f
};
uint8_t cipher2[] =
{
    0xd2, 0x96, 0xcd, 0x94, 0xc2, 0xcc, 0xcf, 0x8a, 0x3a, 0x86, 0x30, 0x28, 0xb5, 0xe1, 0xdc, 0x0a,
    0x75, 0x86, 0x60, 0x2d, 0x25, 0x3c, 0xff, 0xf9, 0x1b, 0x82, 0x66, 0xbe, 0xa6, 0xd6, 0x1a, 0xb1
};

/* Case #3: Encrypting 48 bytes (3 blocks) using AES-CBC with 128-bit key */
uint8_t key3[] = {0x6c, 0x3e, 0xa0, 0x47, 0x76, 0x30, 0xce, 0x21, 0xa2, 0xce, 0x33, 0x4a, 0xa7, 0x46, 0xc2, 0xcd};
uint8_t iv3[] = {0xc7, 0x82, 0xdc, 0x4c, 0x09, 0x8c, 0x66, 0xcb, 0xd9, 0xcd, 0x27, 0xd8, 0x25, 0x68, 0x2c, 0x81};
uint8_t plain3[] = "This is a 48-byte message (exactly 3 AES blocks)";
uint8_t cipher3[] =
{
    0xd0, 0xa0, 0x2b, 0x38, 0x36, 0x45, 0x17, 0x53, 0xd4, 0x93, 0x66, 0x5d, 0x33, 0xf0, 0xe8, 0x86,
    0x2d, 0xea, 0x54, 0xcd, 0xb2, 0x93, 0xab, 0xc7, 0x50, 0x69, 0x39, 0x27, 0x67, 0x72, 0xf8, 0xd5,
    0x02, 0x1c, 0x19, 0x21, 0x6b, 0xad, 0x52, 0x5c, 0x85, 0x79, 0x69, 0x5d, 0x83, 0xba, 0x26, 0x84
};

/* Case #4: Encrypting 64 bytes (4 blocks) using AES-CBC with 128-bit key */
uint8_t key4[] = {0x56, 0xe4, 0x7a, 0x38, 0xc5, 0x59, 0x89, 0x74, 0xbc, 0x46, 0x90, 0x3d, 0xba, 0x29, 0x03, 0x49};
uint8_t iv4[] = {0x8c, 0xe8, 0x2e, 0xef, 0xbe, 0xa0, 0xda, 0x3c, 0x44, 0x69, 0x9e, 0xd7, 0xdb, 0x51, 0xb7, 0xd9};
uint8_t plain4[] =
{
    0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
    0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
    0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
    0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf
};
uint8_t cipher4[] =
{
    0xc3, 0x0e, 0x32, 0xff, 0xed, 0xc0, 0x77, 0x4e, 0x6a, 0xff, 0x6a, 0xf0, 0x86, 0x9f, 0x71, 0xaa,
    0x0f, 0x3a, 0xf0, 0x7a, 0x9a, 0x31, 0xa9, 0xc6, 0x84, 0xdb, 0x20, 0x7e, 0xb0, 0xef, 0x8e, 0x4e,
    0x35, 0x90, 0x7a, 0xa6, 0x32, 0xc3, 0xff, 0xdf, 0x86, 0x8b, 0xb7, 0xb2, 0x9d, 0x3d, 0x46, 0xad,
    0x83, 0xce, 0x9f, 0x9a, 0x10, 0x2e, 0xe9, 0x9d, 0x49, 0xa5, 0x3e, 0x87, 0xf4, 0xc3, 0xda, 0x55
};

uint8_t expanded_key[176];
uint8_t buf[512];
uint8_t iv_enc[16];
uint8_t iv_dec[16];

void log_buffer(const uint8_t buf[], int len)
{
    int i;

    for (i = 0;  i < len;  i++)
        printf(" %02x", buf[i]);
    printf("\n");
}

int main(int argc, char *argv[])
{
    WDTCTL = WDTPW | WDTHOLD;

    memcpy(buf, plain1, 16);
    memcpy(iv_enc, iv1, 16);
    memcpy(iv_dec, iv1, 16);
    aes128_expand_key(expanded_key, key1);
    aes128_encrypt_cbc(buf, iv_enc, expanded_key);
    if (memcmp(buf, cipher1, 16))
    {
        //fprintf(stderr, "Error in test 1 encode\n");
        //exit(2);
        for (;;);
    }
    aes128_decrypt_cbc(buf, iv_dec, expanded_key);
    if (memcmp(buf, plain1, 16))
    {
        //fprintf(stderr, "Error in test 1 decode\n");
        //exit(2);
        for (;;);
    }
    
    memcpy(buf, plain2, 32);
    memcpy(iv_enc, iv2, 16);
    memcpy(iv_dec, iv2, 16);
    aes128_expand_key(expanded_key, key2);
    aes128_encrypt_cbc(buf, iv_enc, expanded_key);
    aes128_encrypt_cbc(buf + 16, iv_enc, expanded_key);
    if (memcmp(buf, cipher2, 32))
    {
        //fprintf(stderr, "Error in test 2 encode\n");
        log_buffer(buf, 32);
        log_buffer(cipher2, 32);
        //exit(2);
        for (;;);
    }
    aes128_decrypt_cbc(buf, iv_dec, expanded_key);
    aes128_decrypt_cbc(buf + 16, iv_dec, expanded_key);
    if (memcmp(buf, plain2, 32))
    {
        //fprintf(stderr, "Error in test 2 decode\n");
        //exit(2);
        for (;;);
    }
    
    memcpy(buf, plain3, 48);
    memcpy(iv_enc, iv3, 16);
    memcpy(iv_dec, iv3, 16);
    aes128_expand_key(expanded_key, key3);
    aes128_encrypt_cbc(buf, iv_enc, expanded_key);
    aes128_encrypt_cbc(buf + 16, iv_enc, expanded_key);
    aes128_encrypt_cbc(buf + 32, iv_enc, expanded_key);
    if (memcmp(buf, cipher3, 48))
    {
        //fprintf(stderr, "Error in test 3 encode\n");
        //exit(2);
        for (;;);
    }
    aes128_decrypt_cbc(buf, iv_dec, expanded_key);
    aes128_decrypt_cbc(buf + 16, iv_dec, expanded_key);
    aes128_decrypt_cbc(buf + 32, iv_dec, expanded_key);
    if (memcmp(buf, plain3, 48))
    {
        //fprintf(stderr, "Error in test 3 decode\n");
        //exit(2);
        for (;;);
    }
    
    memcpy(buf, plain4, 64);
    memcpy(iv_enc, iv4, 16);
    memcpy(iv_dec, iv4, 16);
    aes128_expand_key(expanded_key, key4);
    aes128_encrypt_cbc(buf, iv_enc, expanded_key);
    aes128_encrypt_cbc(buf + 16, iv_enc, expanded_key);
    aes128_encrypt_cbc(buf + 32, iv_enc, expanded_key);
    aes128_encrypt_cbc(buf + 48, iv_enc, expanded_key);
    if (memcmp(buf, cipher4, 64))
    {
        //fprintf(stderr, "Error in test 4 encode\n");
        //exit(2);
        for (;;);
    }
    aes128_decrypt_cbc(buf, iv_dec, expanded_key);
    aes128_decrypt_cbc(buf + 16, iv_dec, expanded_key);
    aes128_decrypt_cbc(buf + 32, iv_dec, expanded_key);
    aes128_decrypt_cbc(buf + 48, iv_dec, expanded_key);
    if (memcmp(buf, plain4, 64))
    {
        //fprintf(stderr, "Error in test 4 decode\n");
        //exit(2);
        for (;;);
    }
    
    //fprintf(stderr, "Tests passed\n");
    return 0;
}
