 /*
* aes128_wrap_tests.c
*
* RFC3394 AES key wrap tests
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
*
*    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


/* RFC3394 AES key wrap tests */

#include <inttypes.h>
#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#if !defined(HOST_BUILD)
#include <msp430.h>
#endif

#include "aes128.h"
#include "aes_wrap.h"

uint8_t common_expanded_aes_gcm_key[176];

uint8_t kek[AES_BLOCK_SIZE];
int keklen;
uint8_t keydata[AES_BLOCK_SIZE];
int keydatalen;
uint8_t cipherdata[AES_BLOCK_SIZE + 8];
int cipherdatalen;

uint8_t common_expanded_aes_wrap_key[176];

void run_test(void)
{
    int i;
    int len;
#if defined(HOST_BUILD)
    uint8_t cipher[keydatalen + 8];
    uint8_t plain[keydatalen];
#else
    uint8_t cipher[AES_BLOCK_SIZE + 8];
    uint8_t plain[AES_BLOCK_SIZE];
#endif

    aes128_expand_key(common_expanded_aes_wrap_key, kek);
    printf("Original:");
    for (i = 0;  i < keydatalen;  i++)
        printf(" %02x", keydata[i]);
    printf("\n");
    len = aes_wrap(cipher, keydata, keydatalen, common_expanded_aes_wrap_key);
    printf("Wrapped len = %d\n", len);
    if (len != keydatalen + 8  ||  len != cipherdatalen)
    {
#if defined(HOST_BUILD)
        printf("Wrapped length mismatch - %d %d\n", keydatalen + 8, len);
        printf("Tests failed\n");
        exit(2);
#else
        for (;;)
            __no_operation();
#endif
    }
#if defined(HOST_BUILD)
    printf("Wrapped:");
    for (i = 0;  i < len;  i++)
        printf(" %02x", cipher[i]);
    printf("\n");
#endif
    for (i = 0;  i < len;  i++)
    {
        if (cipher[i] != cipherdata[i])
        {
#if defined(HOST_BUILD)
            printf("Wrapped data mismatch\n");
            printf("Tests failed\n");
            exit(2);
#else
            for (;;)
                __no_operation();
#endif
        }
    }
    len = aes_unwrap(plain, cipher, len, common_expanded_aes_wrap_key);
    printf("Unwrapped len = %d\n", len);
    if (len != keydatalen)
    {
#if defined(HOST_BUILD)
        printf("Unwrapped length mismatch - %d %d\n", keydatalen, len);
        printf("Tests failed\n");
        exit(2);
#else
        for (;;)
            __no_operation();
#endif
    }
#if defined(HOST_BUILD)
    printf("Unwrapped:");
    for (i = 0;  i < len;  i++)
        printf(" %02x", plain[i]);
    printf("\n");
#endif
    for (i = 0;  i < len;  i++)
    {
        if (plain[i] != keydata[i])
        {
#if defined(HOST_BUILD)
            printf("Unwrapped data mismatch\n");
            printf("Tests failed\n");
            exit(2);
#else
            for (;;)
                __no_operation();
#endif
        }
    }
}

int main(int argc, char *argv[])
{
#if !defined(HOST_BUILD)
    WDTCTL = WDTPW | WDTHOLD;
#endif

    /* This is the test case from RFC3394 */
    keklen = AES_BLOCK_SIZE;
    memcpy(kek,
           "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
           keklen);
    keydatalen = AES_BLOCK_SIZE;
    memcpy(keydata,
           "\x00\x11\x22\x33\x44\x55\x66\x77\x88\x99\xAA\xBB\xCC\xDD\xEE\xFF",
           keydatalen);
    cipherdatalen = AES_BLOCK_SIZE + 8;
    memcpy(cipherdata,
           "\x1F\xA6\x8B\x0A\x81\x12\xB4\x47\xAE\xF3\x4B\xD8\xFB\x5A\x7B\x82\x9D\x3E\x86\x23\x71\xD2\xCF\xE5",
           cipherdatalen);

    run_test();

#if defined(HOST_BUILD)
    printf("Tests passed\n");
#endif
    return 0;
}
